<?php

namespace Drupal\entity_display_json\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\paragraphs\Entity\Paragraph;
use Entity;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Returns responses for Entity Display JSON routes.
 */
class EntityDisplayJsonController extends ControllerBase {

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * JsonController constructor.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager) {
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_field.manager')
    );
  }

  /**
   * Get a paragraph and view mode and display it as JSON.
   * {@inheritdoc}
   */
  public static function process_entity($entity, $display_id = 'default') {
    $entity_type = $entity->getEntityTypeId();
    $bundle = $entity->bundle();
    $fields = \Drupal::service('entity_display.repository')->getViewDisplay($entity_type, $bundle, $display_id)->getComponents();
    $data = [];
    $data['entity_type'] = $entity_type;
    $data['bundle'] = $bundle;
    $data['id'] = $entity->id();
    $data['uuid'] = $entity->uuid();
    foreach ($fields as $field_name => $field_value) {
      if (!$field_value['label']) {
        continue;
      }
      // Entity reference fields.
      if ($entity->$field_name->target_id) {
        $ref_entities = $entity->$field_name->referencedEntities();
        $display_id = $field_value["settings"]["view_mode"];
        foreach ($ref_entities as $ref_entity) {
          if ($field_name == 'thumbnail') {
            $image_style = $field_value["settings"]["image_style"];
            $imageUri = $ref_entity->getFileUri();
            $data[$field_name][] = \Drupal\image\Entity\ImageStyle::load($image_style)->buildUrl($imageUri);
            continue;
          }
          $data[$field_name][] = self::process_entity($ref_entity, $display_id);
        }
      }
      if ($entity->$field_name->value) {
        $data[$field_name] = $entity->$field_name->value;
      }
    }
    return $data;
  }
  /**
   * Generates the JSON response based on entity display settings.
   *
   * @param string $entity_id
   *   The entity ID.
   * @param string $display_id
   *   The display ID.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   Thrown when the entity or display cannot be found.
   */
  public function build($entity_type, $uuid, $display_id = 'default') {

    $entity = reset(\Drupal::service('entity_type.manager')->getStorage($entity_type)->loadByProperties(['uuid' => $uuid]));
    $data = $this->process_entity($entity, $display_id);

    // Return the JSON response.
    return new JsonResponse($data);
  }
}
